﻿using InstanceFactory.MimicAppConfig.Common;
using Windows.Storage;

namespace InstanceFactory.MimicAppConfig.Configuration
{
  /// <summary>
  /// Handles all the configuration stuff.
  /// </summary>
  internal class AppConfiguration : BindableBase
  {
    #region Private Const Data Member

    /// <summary>
    /// Keeps the name of roaming settings key for using the relese app.config file.
    /// </summary>
    private const string UseReleaseConfigKey = "UseReleaseConfig";

    #endregion Private Const Data Member


    #region Private Data Member

    /// <summary>
    /// The flag that indicates whether the release app.config file should be used.
    /// </summary>
    private bool m_UseReleaseConfig;

    #endregion Private Data Member


    #region Private Static Data Member

    /// <summary>
    /// Keeps the one and only Instance of this class.
    /// </summary>
    private static AppConfiguration s_Instance = new AppConfiguration();

    #endregion Private Static Data Member


    #region Public Properties

    /// <summary>
    /// Gets / sets the flag that indicates whether the release app.config file should be used.
    /// </summary>
    public bool UseReleaseConfig
    {
      get { return (m_UseReleaseConfig); }
      set { SetProperty(ref m_UseReleaseConfig, value); }
    }

    #endregion Public Properties


    #region Public Static Properties

    /// <summary>
    /// Provides access to the app's configuration.
    /// </summary>
    public static AppConfiguration Current { get { return (s_Instance); } }

    #endregion Public Static Properties


    #region Private Constructors

    /// <summary>
    /// Initializes a new instance of <see cref="AppConfiguration"/>.
    /// </summary>
    private AppConfiguration()
    {
      // Set default value.
      UseReleaseConfig = true;

      // Read it from the roaming settings
      ApplicationDataContainer roamingSettings = GetRoamingSettings();

      // Get the config value
      UseReleaseConfig = (bool)roamingSettings.Values[AppConfiguration.UseReleaseConfigKey];
    }

    #endregion Private Constructors


    #region Private Methods

    /// <summary>
    /// Returns the roaming settings with all keys needed for this app.
    /// </summary>
    /// <returns>Roaming Settings.</returns>
    private ApplicationDataContainer GetRoamingSettings()
    {
      // Get the roaming settings
      ApplicationDataContainer roamingSettings = ApplicationData.Current.RoamingSettings;

      // If the key does not exits, create it with default value.
      if (!roamingSettings.Values.ContainsKey(AppConfiguration.UseReleaseConfigKey))
      {
        roamingSettings.Values.Add(AppConfiguration.UseReleaseConfigKey, true);
      }

      // Return the settings
      return (roamingSettings);
    }

    #endregion Private Methods


    #region Public Methods

    /// <summary>
    /// Saves the configuration.
    /// </summary>
    public void Save()
    {
      // Get the roaming settings
      ApplicationDataContainer roamingSettings = GetRoamingSettings();

      // Set the values in RoamingSettings; that's it. Properties are set via DataBinding
      roamingSettings.Values[AppConfiguration.UseReleaseConfigKey] = UseReleaseConfig;
    }

    #endregion Public Methods
  }
}